@@ -4,7 +4,7 @@ import tinychain.error
 import tinychain.graph
 import tinychain.graph.edge
 import tinychain.host
-import tinychain.ml.conv
+import tinychain.ml.cnn
 import tinychain.ml.dnn
 import tinychain.op
 import tinychain.ref
@@ -12,8 +12,9 @@ import tinychain.ref
 from tinychain.cluster import Cluster, write_cluster
 from tinychain.collection import Column
 from tinychain.collection import btree, table, tensor
-from tinychain.decorators import attribute, closure, delete_method, get_method, post_method, put_method, delete_op, get_op, post_op, put_op
-from tinychain.ref import After, Before, Case, If, New, While
+from tinychain.decorators import closure, delete_method, get_method, post_method, put_method, delete_op, get_op, post_op, put_op
+from tinychain.ml import linalg
+from tinychain.ref import After, Case, If, While
 from tinychain.reflect import Meta, Object
 from tinychain.state import Class, Instance, Map, State, Scalar, Stream, Tuple
 from tinychain.value import *
