@@ -3,7 +3,7 @@
 from tinychain import ref
 from tinychain.state import Map, State, Stream, Tuple
 from tinychain.util import form_of, to_json, uri, URI
-from tinychain.value import Bool, I16, F32, Number
+from tinychain.value import Bool, I16, F32, Number, UInt
 
 from .bound import Range
 from .collection import Collection
@@ -23,12 +23,21 @@ class Schema(object):
     def __json__(self):
         return to_json([self.shape, self.dtype])
 
+    def __repr__(self):
+        return f"tensor schema with shape {self.shape} and dtype {self.dtype}"
+
 
 class Tensor(Collection):
     """An n-dimensional array of numbers."""
 
     __uri__ = uri(Collection) + "/tensor"
 
+    @classmethod
+    def create(cls, shape, dtype=F32):
+        """Create a new, empty `Tensor`. Call this method to initialize a persistent `Tensor` in a `Chain`."""
+
+        return cls(ref.Get(cls, Schema(shape, dtype)))
+
     @classmethod
     def load(cls, shape, dtype, data):
         """
@@ -47,12 +56,17 @@ class Tensor(Collection):
         return super().load(Schema(shape, dtype), data)
 
     def __getitem__(self, bounds):
+        parent = self
         bounds = _handle_bounds(bounds)
-        return self._get("", bounds, Tensor)
+
+        class Slice(self.__class__):
+            def write(self, value):
+                return parent._put("", bounds, value)
+
+        return self._get("", bounds, Slice)
 
     def __setitem__(self, bounds, value):
-        bounds = _handle_bounds(bounds)
-        return self._put("", bounds, value)
+        raise NotImplementedError("use Tensor.write instead")
 
     def __add__(self, other):
         return self.add(other)
@@ -107,6 +121,16 @@ class Tensor(Collection):
 
         return self._get("abs", rtype=self.__class__)
 
+    def acos(self):
+        """Return the element-wise arccosine of this `Tensor`."""
+
+        return self._get("acos", rtype=self.__class__)
+
+    def acosh(self):
+        """Return the element-wise hyperbolic arccosine of this `Tensor`."""
+
+        return self._get("acosh", rtype=self.__class__)
+
     def add(self, other):
         """Return the element-wise sum of this `Tensor` and another `Tensor` or `Number`."""
 
@@ -122,6 +146,26 @@ class Tensor(Collection):
 
         return self._get("any", rtype=Bool)
 
+    def asin(self):
+        """Return the element-wise arcsine of this `Tensor`."""
+
+        return self._get("asin", rtype=self.__class__)
+
+    def asinh(self):
+        """Return the element-wise hyperbolic arcsine of this `Tensor`."""
+
+        return self._get("asinh", rtype=self.__class__)
+
+    def atan(self):
+        """Return the element-wise arctangent of this `Tensor`."""
+
+        return self._get("atan", rtype=self.__class__)
+
+    def atanh(self):
+        """Return the element-wise hyperbolic arctangent of this `Tensor`."""
+
+        return self._get("atanh", rtype=self.__class__)
+
     def cast(self, number_type):
         """Cast the data type of `Tensor` into the given `number_type`."""
 
@@ -132,11 +176,26 @@ class Tensor(Collection):
 
         return self.__class__(ref.Post(uri(Tensor) + "/copy_from", {"tensor": self}))
 
+    def cos(self):
+        """Return the element-wise cosine of this `Tensor`."""
+
+        return self._get("cos", rtype=self.__class__)
+
+    def cosh(self):
+        """Return the element-wise hyperbolic cosine of this `Tensor`."""
+
+        return self._get("cosh", rtype=self.__class__)
+
     def div(self, other):
         """Divide this `Tensor` by another `Tensor` or `Number`, broadcasting if necessary."""
 
         return self._post("div", Map(r=other), Tensor)
 
+    def flip(self, axis):
+        """Flip the elements in this `Tensor` along the specified `axis`."""
+
+        return self._get("flip", axis, Tensor)
+
     def eq(self, other):
         """Return a boolean `Tensor` with element-wise equality values."""
 
@@ -192,6 +251,12 @@ class Tensor(Collection):
 
         return self._post("xor", Map(r=other), Tensor)
 
+    @property
+    def ndim(self):
+        """Return the number of dimensions of this `Tensor`."""
+
+        return self._get("ndim", rtype=UInt)
+
     def mul(self, other):
         """Multiply this `Tensor` by another `Tensor` or `Number`, broadcasting if necessary."""
 
@@ -213,10 +278,42 @@ class Tensor(Collection):
         rtype = Number if axis is None else self.__class__
         return self._get("product", axis, rtype)
 
+    def reshape(self, shape):
+        """Return a view of this `Tensor` with the given `shape`."""
+
+        return self._get("reshape", shape, self.__class__)
+
+    # TODO: use a custom shape object to support `type(x.shape[0]) == UInt`
+    @property
     def shape(self):
         """Return the shape of this `Tensor`."""
+
         return self._get("shape", rtype=Tuple)
 
+    def sin(self):
+        """Return the element-wise sine of this `Tensor`."""
+
+        return self._get("sin", rtype=self.__class__)
+
+    def sinh(self):
+        """Return the element-wise hyperbolic sine of this `Tensor`."""
+
+        return self._get("sinh", rtype=self.__class__)
+
+    def split(self, num_or_size_splits, axis=0):
+        """
+        Split this `Tensor` into multiple slices along the given `axis`.
+
+        If `num_or_size_splits` is a `Number`, the `tensor` will be sliced along `axis` `num_or_size_splits` times;
+        if `self.shape[axis] % num_or_size_splits != 0` then a `BadRequest` error will be raised.
+
+        If `num_or_size_splits` is a `Tuple` with length `n` then the `tensor` will be split into `n` new `Tensor` s
+        each with `shape[axis] == num_or_size_splits[axis]`; if the sum of `num_or_size_splits` is not equal to
+        `self.shape[axis]` then a `BadRequest` error will be raised.
+        """
+
+        return self._get("split", (num_or_size_splits, axis), self.__class__)
+
     def sub(self, other):
         """Subtract another `Tensor` or `Number` from this one, broadcasting if necessary."""
 
@@ -228,6 +325,16 @@ class Tensor(Collection):
         rtype = Number if axis is None else self.__class__
         return self._get("sum", axis, rtype)
 
+    def tan(self):
+        """Return the element-wise tangent of this `Tensor`."""
+
+        return self._get("tan", rtype=self.__class__)
+
+    def tanh(self):
+        """Return the element-wise hyperbolic tangent of this `Tensor`."""
+
+        return self._get("tanh", rtype=self.__class__)
+
     def transpose(self, permutation=None):
         """
         Return a view of this `Tensor` with its axes transposed according to the given permutation.
@@ -237,15 +344,10 @@ class Tensor(Collection):
 
         return self._get("transpose", permutation, self.__class__)
 
-    def write(self, bounds, value):
-        """Write a `Tensor` or `Number` to the given slice of this one, broadcasting if needed."""
+    def write(self, value):
+        """Overwrite this `Tensor` with the given `Tensor` or `Number`, broadcasting if needed."""
 
-        return self.__setitem__(bounds, value)
-
-    def overwrite(self, value):
-        """Overwrite all elements of this `Tensor` with the given `value`, broadcasting if needed."""
-
-        return self.__setitem__(None, value)
+        return self._put("", None, value)
 
 
 class Dense(Tensor):
@@ -268,6 +370,16 @@ class Dense(Tensor):
 
         return cls(ref.Get(uri(cls) + "/range", (shape, start, stop)))
 
+    @classmethod
+    def concatenate(cls, tensors, axis=None):
+        """Create a new `Dense` tensor by concatenating the given `tensors` along the given `axis`."""
+
+        params = {"tensors": tensors}
+        if axis:
+            params["axis"] = axis
+
+        return cls(ref.Post(uri(cls) + "/concatenate", params))
+
     @classmethod
     def constant(cls, shape, value):
         """Return a `Dense` tensor filled with the given `value`."""
@@ -300,6 +412,11 @@ class Dense(Tensor):
         bounds = _handle_bounds(bounds)
         return self._get("elements", bounds, Stream)
 
+    def as_sparse(self):
+        """Return a :class:`Sparse` view of this `Dense` tensor."""
+
+        return self._get("sparse", rtype=Sparse)
+
 
 class Sparse(Tensor):
     """
@@ -331,20 +448,36 @@ class Sparse(Tensor):
         bounds = _handle_bounds(bounds)
         return self._get("elements", bounds, Stream)
 
+    def as_dense(self):
+        """Return a :class:`Dense` view of this `Sparse` tensor."""
+
+        return self._get("dense", rtype=Dense)
+
 
 def einsum(format, tensors):
     """
     Return the Einstein summation of the given `tensors` according the the given `format` string.
 
-    Example: `einsum("ij,kj->ik", [a, b]) # compute the dot product of a and b`
+    Example: `einsum("ij,jk->ik", [a, b]) # multiply two matrices`
 
-    The tensor product is computed from left to right, so when using any `Sparse` tensors, it's important to put
-    the sparsest first in the list to avoid redundant broadcasting.
+    The tensor product is computed from left to right, so when using any `Sparse` tensors,
+    it's important to put the sparsest first in the list to avoid redundant broadcasting.
     """
 
     return Tensor(ref.Post(uri(Tensor) + "/einsum", {"format": format, "tensors": tensors}))
 
 
+def tile(tensor, multiples):
+    """Construct a new `Tensor` by tiling the given `tensor` `multiples` times.
+
+    The values of `tensor` are repeated `multiples[x]` times along the `x`th axis of the output.
+    `multiples` must be a positive integer or a `Tuple` of length `tensor.ndim`.
+    """
+
+    rtype = tensor.__class__ if isinstance(tensor, Tensor) else Tensor
+    return rtype(ref.Post(uri(Tensor) + "/tile", {"tensor": tensor, "multiples": multiples}))
+
+
 def _handle_bounds(bounds):
     if bounds is None or isinstance(bounds, ref.Ref) or isinstance(bounds, URI):
         return bounds
